@@ -1,6 +1,6 @@
 /**
  * perl-libxml-mm.c
- * $Id: perl-libxml-mm.c,v 1.32 2002/10/25 10:13:28 phish Exp $
+ * $Id: perl-libxml-mm.c,v 1.36 2003/08/18 08:40:19 phish Exp $
  *
  * Basic concept:
  * perl varies in the implementation of UTF8 handling. this header (together
@@ -133,7 +133,7 @@ typedef ProxyNode* ProxyNodePtr;
 
 #define PmmENCODING(node)    node->encoding
 #define PmmNodeEncoding(node) ((ProxyNodePtr)(node->_private))->encoding
-
+#define PmmDocEncoding(node) (node->charset)
 /* creates a new proxy node from a given node. this function is aware
  * about the fact that a node may already has a proxy structure.
  */
@@ -143,7 +143,7 @@ PmmNewNode(xmlNodePtr node)
     ProxyNodePtr proxy = NULL;
 
     if ( node == NULL ) {
-        warn( "no node found\n" );
+        xs_warn( "no node found\n" );
         return NULL;
     }
 
@@ -174,9 +174,12 @@ PmmNewFragment(xmlDocPtr doc)
     frag   = xmlNewDocFragment( doc );
     retval = PmmNewNode(frag);
 
-    if ( doc ) {
+    if ( doc != NULL ) {
         xs_warn("inc document\n");
-        PmmREFCNT_inc(((ProxyNodePtr)doc->_private));
+        /* under rare circumstances _private is not set correctly? */
+        if ( doc->_private != NULL ) {
+            PmmREFCNT_inc(((ProxyNodePtr)doc->_private));
+        }
         retval->owner = (xmlNodePtr)doc;
     }
 
@@ -204,7 +207,7 @@ PmmFreeNode( xmlNodePtr node )
         }
         break;
     case XML_DTD_NODE:
-        if ( node->doc ) {
+        if ( node->doc != NULL ) {
             if ( node->doc->extSubset != (xmlDtdPtr)node 
                  && node->doc->intSubset != (xmlDtdPtr)node ) {
                 xs_warn( "PFN: XML_DTD_NODE\n");
@@ -234,7 +237,7 @@ PmmREFCNT_dec( ProxyNodePtr node )
     ProxyNodePtr owner = NULL;  
     int retval = 0;
 
-    if ( node ) {
+    if ( node != NULL ) {
         retval = PmmREFCNT(node)--;
         if ( PmmREFCNT(node) <= 0 ) {
             xs_warn( "NODE DELETATION\n" );
@@ -306,7 +309,7 @@ PmmNodeToSv( xmlNodePtr node, ProxyNodePtr owner )
         xs_warn(" return new perl node\n");
         xs_warn( CLASS );
 
-        if ( node->_private ) {
+        if ( node->_private != NULL ) { 
             dfProxy = PmmNewNode(node);
         }
         else {
@@ -404,8 +407,10 @@ PmmSvNodeExt( SV* perlnode, int copy )
         xs_warn("   perlnode found\n" );
         if ( sv_derived_from(perlnode, "XML::LibXML::Node")  ) {
             proxy = SvPROXYNODE(perlnode);
-            xs_warn( "is a xmlNodePtr structure\n" );
-            retval = PmmNODE( proxy ) ;
+            if ( proxy != NULL ) {
+                xs_warn( "is a xmlNodePtr structure\n" );
+                retval = PmmNODE( proxy ) ;
+            }
 
             if ( retval != NULL
                  && ((ProxyNodePtr)retval->_private) != proxy ) {
@@ -466,7 +471,7 @@ PmmSetSvOwner( SV* perlnode, SV* extra )
 void
 PmmFixOwnerList( xmlNodePtr list, ProxyNodePtr parent )
 {
-    if ( list ) {
+    if ( list != NULL ) {
         xmlNodePtr iterator = list;
         while ( iterator != NULL ) {
             switch ( iterator->type ) {
@@ -527,7 +532,7 @@ PmmFixOwner( ProxyNodePtr nodetofix, ProxyNodePtr parent )
             break;
         }
 
-        if ( PmmOWNER(nodetofix) ) {
+        if ( PmmOWNER(nodetofix) != NULL ) {
             oldParent = PmmOWNERPO(nodetofix);
         }
         
@@ -545,7 +550,7 @@ PmmFixOwner( ProxyNodePtr nodetofix, ProxyNodePtr parent )
                 PmmOWNER(nodetofix) = NULL;
             }
             
-            if ( oldParent && oldParent != nodetofix )
+            if ( oldParent != NULL && oldParent != nodetofix )
                 PmmREFCNT_dec(oldParent);
             
             if ( PmmNODE(nodetofix)->type != XML_ATTRIBUTE_NODE
@@ -604,7 +609,7 @@ PmmContextREFCNT_dec( ProxyNodePtr node )
 { 
     xmlParserCtxtPtr libnode = NULL;
     int retval = 0;
-    if ( node ) {
+    if ( node != NULL ) {
         retval = PmmREFCNT(node)--;
         if ( PmmREFCNT(node) <= 0 ) {
             xs_warn( "NODE DELETATION\n" );
@@ -715,7 +720,7 @@ PmmFastEncodeString( int charset,
             /* warn( "encoded string is %s" , retval); */
         }
         else {
-            xs_warn( "b0rked encoiding!\n");
+            /* warn( "b0rked encoiding!\n"); */
         }
         
         xmlBufferFree( in );
@@ -740,11 +745,9 @@ PmmFastDecodeString( int charset,
     }
 
     if ( charset > 1 ) {
-        /* warn( "use document encoding %s", encoding ); */
         coder= xmlGetCharEncodingHandler( charset );
     }
     else if ( charset == XML_CHAR_ENCODING_ERROR ){
-        /* warn("no standard encoding\n"); */
         coder = xmlFindCharEncodingHandler( (const char *) encoding );
     }
     else {
@@ -758,7 +761,7 @@ PmmFastDecodeString( int charset,
         
         xmlBufferCat( in, string );        
         if ( xmlCharEncOutFunc( coder, out, in ) >= 0 ) {
-            retval = xmlStrdup(out->content);
+            retval = xmlCharStrndup(xmlBufferContent(out), xmlBufferLength(out));
         }
         else {
             xs_warn("decoding error \n");
@@ -804,7 +807,6 @@ char*
 PmmDecodeString( const char *encoding, const xmlChar *string){
     char *ret=NULL;
     xmlCharEncoding enc;
-    xmlBufferPtr in = NULL, out = NULL;
     xmlCharEncodingHandlerPtr coder = NULL;
 
     if ( string != NULL ) {
@@ -912,9 +914,9 @@ nodeC2Sv( const xmlChar * string,  xmlNodePtr refnode )
 
     if ( refnode != NULL ) {
         xmlDocPtr real_doc = refnode->doc;
-        if ( real_doc && real_doc->encoding != NULL ) {
+        if ( real_doc != NULL && real_doc->encoding != NULL ) {
 
-            xmlChar * decoded = PmmFastDecodeString( PmmNodeEncoding(real_doc) ,
+            xmlChar * decoded = PmmFastDecodeString( PmmDocEncoding(real_doc) ,
                                                      (const xmlChar *)string,
                                                      (const xmlChar*)real_doc->encoding);
             len = xmlStrlen( decoded );
@@ -959,7 +961,7 @@ nodeSv2C( SV * scalar, xmlNodePtr refnode )
        perl. in this cases the library assumes, all strings are in
        UTF8. if a programmer likes to have the intelligent code, he
        needs to upgrade perl */
-#ifdef HAVE_UTF8        
+
     if ( refnode != NULL ) {
         xmlDocPtr real_dom = refnode->doc;
         xs_warn("have node!");
@@ -975,15 +977,18 @@ nodeSv2C( SV * scalar, xmlNodePtr refnode )
                     xs_warn( "no undefs" );
 #ifdef HAVE_UTF8
                     xs_warn( "use UTF8" );
-                    if( !DO_UTF8(scalar) && real_dom->encoding != NULL ) {
+                    if( !DO_UTF8(scalar) && real_dom != NULL && real_dom->encoding != NULL ) {
                         xs_warn( "string is not UTF8\n" );
 #else
-                    if ( real_dom->encoding != NULL ) {        
+                    if ( real_dom != NULL && real_dom->encoding != NULL ) {        
 #endif
                         xs_warn( "domEncodeString!" );
-                        ts= PmmFastEncodeString( PmmNodeEncoding(real_dom),
+                      /*  if ( string == NULL || *string == 0 ) warn("string is empty" ); */
+                       
+                        ts= PmmFastEncodeString( PmmDocEncoding(real_dom),
                                                  string,
                                                  (const xmlChar*)real_dom->encoding );
+
                         xs_warn( "done!" );
                         if ( string != NULL ) {
                             xmlFree(string);
@@ -994,7 +999,7 @@ nodeSv2C( SV * scalar, xmlNodePtr refnode )
                         xs_warn( "no encoding set, use UTF8!\n" );
                     }
                 }
-                if ( string == NULL ) xs_warn( "string is NULL\n" );
+                /* if ( string == NULL ) warn( "string is NULL\n" ); */
                 return string;
             }
             else {
@@ -1007,7 +1012,6 @@ nodeSv2C( SV * scalar, xmlNodePtr refnode )
         }
     }
     xs_warn("no encoding !!");
-#endif
 
     return  Sv2C( scalar, NULL ); 
 }
